package com.galaxis.manatee.dao;


import com.galaxis.capsule.util.GalaxisRepository;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunMemberCostDO;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author kxuan
 */
@Repository
public interface ChuanyunMemberCostDao extends GalaxisRepository<ChuanyunMemberCostDO,String> {

    /**
     * 获取项目人员总工时总成本
     * @return
     */
    @Query(value="SELECT chuanyun_self_cost.object_id," +
            "SUM(chuanyun_self_cost.total_hour) AS total_hour," +
            "SUM(chuanyun_self_cost.cost) AS cost," +
            "chuanyun_self_cost.project_type," +
            "chuanyun_self_cost.project_id," +
            "chuanyun_self_cost.status," +
            "chuanyun_self_cost.user_id," +
            "chuanyun_self_cost.monthly_total_hour AS person_days"+
            " FROM chuanyun_self_cost " +
            "GROUP BY chuanyun_self_cost.project_id," +
            "chuanyun_self_cost.status," +
            "chuanyun_self_cost.user_id",
            nativeQuery = true)
    List<ChuanyunMemberCostDO> getMemberCost();

    /**
     * 根据项目ID和人员ID获取工时成本数据
     * @param userId    用户ID
     * @param projectId 项目Id
     * @param status    项目流程状态
     * @return  人员-项目工时
     */
    ChuanyunMemberCostDO findByUserIdAndProjectIdAndStatus(String userId,String projectId,String status);

    /**
     * 删除审核中数据
     * @param status
     */
    @Transactional
    void deleteAllByStatus(String status);
}
