package com.galaxis.manatee.dao;

import com.galaxis.capsule.util.GalaxisRepository;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunSelfCostDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunSelfWorkHourDO;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Repository
public interface ChuanyunSelfCostDao extends GalaxisRepository<ChuanyunSelfCostDO,String> {

    /**
     * 根据状态查询月工时统计记录
     * @param status    状态
     * @param userId    用户Id
     * @return  月工时统计列表
     */
    List<ChuanyunSelfCostDO> findAllByStatusAndUserId(String status,String userId);

    /**
     * 获取项目人员总工时
     * @return  月工时统计列表
     */
    @Query(value="SELECT chuanyun_self_cost.object_id,SUM(chuanyun_self_cost.total_hour) AS total_hour,SUM(chuanyun_self_cost.cost) AS cost,chuanyun_self_cost.project_id,chuanyun_self_cost.user_id"+
            " FROM chuanyun_self_cost GROUP BY chuanyun_self_cost.project_id,chuanyun_self_cost.user_id",nativeQuery = true)
    List<ChuanyunSelfCostDO> getMemberCost();

    /**
     * 获取最近一个月工时
     * @param pagerequest
     * @return
     */
    @Query(value="SELECT * FROM chuanyun_self_cost WHERE MONTH(chuanyun_self_cost.day_log_date)>=(MONTH(NOW())-1) AND YEAR(chuanyun_self_cost.day_log_date)=YEAR(NOW())"
            ,countQuery="SELECT COUNT(1) FROM chuanyun_self_cost WHERE MONTH(chuanyun_self_cost.day_log_date)>=(MONTH(NOW())-1) AND YEAR(chuanyun_self_cost.day_log_date)=YEAR(NOW())",nativeQuery = true)
    Page<ChuanyunSelfCostDO> getRecentlyMonthHour(PageRequest pagerequest);

    /**
     * 根据用户ID和项目ID，日期获取工时成本数据
     * @param userId    用户Id
     * @param projectId 项目Id
     * @param status    工时状态
     * @param dayLogDate    日期
     * @return  月工时统计列表
     */
    @Query(value="SELECT * FROM chuanyun_self_cost " +
            "WHERE user_id = ?1 " +
            "AND project_id = ?2 " +
            "AND project_type=?3" +
            " AND DATE_FORMAT(chuanyun_self_cost.day_log_date,'%Y-%m')=DATE_FORMAT(?4,'%Y-%m') AND status=?5",nativeQuery = true)
    ChuanyunSelfCostDO getOldSelfDO(@Param("userId")String userId,@Param("projectId")String projectId,@Param("projectType") String projectType,@Param("dayLogDate")String dayLogDate,String status);
}
