package com.galaxis.manatee.dao;

import com.galaxis.capsule.util.GalaxisRepository;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunProjectCostDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunWeekMemberHourDO;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Repository
public interface ChuanyunWeekMemberHourDao extends GalaxisRepository<ChuanyunWeekMemberHourDO,String> {

    /**
     * 根据状态删除数据
     * @param status    状态
     * @return  删除条数
     */
    @Transactional
    Integer deleteAllByStatus(String status);

    /**
     * 人员-项目-周度工时汇总
     * @return
     */
    @Query(value="SELECT chuanyun_self_work_hour.object_id," +
            "DATE_FORMAT(day_log_date,'%Y') as year," +
            "WEEKOFYEAR(chuanyun_self_work_hour.day_log_date) AS week," +
            "project_id,user_id," +
            "SUM(standard_work_hour)AS standard_work_hour," +
            "project_type," +
            "`status`" +
            "FROM chuanyun_self_work_hour " +
            "WHERE project_id !='' and LENGTH(user_id) = CHAR_LENGTH(user_id)" +
            "GROUP BY chuanyun_self_work_hour.project_id," +
            "chuanyun_self_work_hour.project_type," +
            "chuanyun_self_work_hour.user_id," +
            "DATE_FORMAT(day_log_date,'%Y')," +
            "WEEKOFYEAR(chuanyun_self_work_hour.day_log_date)," +
            "chuanyun_self_work_hour.`status`",nativeQuery = true)
    List<ChuanyunWeekMemberHourDO> getWeekMemberHour();

    /**
     * 获取最近一个月周工时
     * @return
     */
    @Query(value="SELECT chuanyun_self_work_hour.object_id," +
            "DATE_FORMAT(day_log_date,'%Y') as year," +
            "WEEKOFYEAR(chuanyun_self_work_hour.day_log_date) AS week," +
            "project_id,user_id," +
            "SUM(standard_work_hour)AS standard_work_hour," +
            "project_type," +
            "`status`" +
            "FROM chuanyun_self_work_hour " +
            "WHERE project_id !='' " +
            "AND WEEKOFYEAR(chuanyun_self_work_hour.day_log_date)>=(WEEKOFYEAR(NOW())-4)" +
            "AND DATE_FORMAT(day_log_date,'%Y')=YEAR(NOW()) " +
            "and LENGTH(user_id) = CHAR_LENGTH(user_id) " +
            "GROUP BY chuanyun_self_work_hour.project_id," +
            "chuanyun_self_work_hour.project_type," +
            "chuanyun_self_work_hour.user_id," +
            "DATE_FORMAT(day_log_date,'%Y')," +
            "WEEKOFYEAR(chuanyun_self_work_hour.day_log_date)," +
            "chuanyun_self_work_hour.`status`",nativeQuery = true)
    List<ChuanyunWeekMemberHourDO> getWeekMemberHourRecently();


    /**
     * 获取最近一个月周工时
     * @param
     * @param pagerequest
     * @return
     */
    @Query(value="SELECT * FROM chuanyun_week_member_hour WHERE chuanyun_week_member_hour.`week` >= (WEEKOFYEAR(NOW())-4) AND chuanyun_week_member_hour.`year`=YEAR(NOW())",
            countQuery="SELECT COUNT(1) FROM chuanyun_week_member_hour WHERE chuanyun_week_member_hour.`week` >= (WEEKOFYEAR(NOW())-4) AND chuanyun_week_member_hour.`year`=YEAR(NOW())",nativeQuery = true)
    Page<ChuanyunWeekMemberHourDO> getRecentlyWeekHour(PageRequest pagerequest);

    /**
     * 判断是否有流程中数据
     * @return
     */
    @Query(value="SELECT chuanyun_self_work_hour.object_id," +
            "DATE_FORMAT(day_log_date,'%Y') as year," +
            "WEEKOFYEAR(chuanyun_self_work_hour.day_log_date) AS week," +
            "project_id,user_id," +
            "SUM(standard_work_hour)AS standard_work_hour," +
            "project_type," +
            "`status`" +
            "FROM chuanyun_self_work_hour " +
            "WHERE project_id=?1 and user_id=?2 and DATE_FORMAT(day_log_date,'%Y')=?3 and " +
            "WEEKOFYEAR(chuanyun_self_work_hour.day_log_date)=?4 and status=2 and project_type=?5" +
            " GROUP BY chuanyun_self_work_hour.project_id," +
            "chuanyun_self_work_hour.project_type," +
            "chuanyun_self_work_hour.user_id," +
            "DATE_FORMAT(day_log_date,'%Y')," +
            "WEEKOFYEAR(chuanyun_self_work_hour.day_log_date)," +
            "chuanyun_self_work_hour.`status`",nativeQuery = true)
    ChuanyunWeekMemberHourDO getProcessWeekHour(@Param("projectId")String projectId,@Param("userId")String userId,@Param("year")String year,@Param("week")String week,@Param("projectType")String projectType);

    /**
     * 项目人员周度是否存在
     * @param projectId
     * @param userId
     * @param week
     * @return
     */
    ChuanyunWeekMemberHourDO findByProjectIdAndUserIdAndWeekAndStatusAndProjectTypeAndYear(String projectId,String userId,String week,String status,String projectType,String year);

    /**
     * 删除状态为审核中的数据
     */
    @Transactional
    void deleteByProjectIdAndUserIdAndYearAndWeekAndStatusAndProjectType(String projectId,String userId,String year,String week,String status,String projectType);
}
