package com.galaxis.manatee.entity.chuanyun.data.object;

import com.dingtalk.api.response.OapiProcessinstanceGetResponse;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.extern.slf4j.Slf4j;
import net.sf.json.JSONArray;
import org.springframework.beans.BeanUtils;
import javax.persistence.Entity;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.UUID;

/**
 * Created by IntelliJ IDEA.
 * User: xuj
 * Date: 2020/7/23
 * @author xuj
 */
@Slf4j
@EqualsAndHashCode(callSuper = true)
@Data
@Entity(name = "CHUANYUN_ASKFORLEAVE")
@JsonIgnoreProperties(ignoreUnknown = true)
public class ChuanyunAskForLeaveDO extends BasicDO {

    /**
     * 表名
     */
    public static final String SCHEMA_CODE ="D001789c490c2213711480d8b2c3eee7099e39d";

    /**
     * 标题
     */
    @JsonProperty("F0000001")
    private String title;

    /**
     * 抄送人
     */
    @JsonProperty("cc_userids")
    private String ccUserids;

    /**
     * 操作
     */
    @JsonProperty("biz_action")
    private String bizAction;

    /**
     * 创建时间
     */
    @JsonProperty("create_time")
    private LocalDateTime createTime;

    /**
     * 结束时间
     */
    @JsonProperty("finish_time")
    private LocalDateTime finishTime;


    @JsonProperty("main_process_instance_id")
    private String mainProcessInstanceId;

    @JsonProperty("is_canncel")
    private int isCanncel;

    /**
     * 发起人userId
     */
    @JsonProperty("originator_userid")
    private String originatorUserid;

    /**
     * 发起人姓名
     */
    @JsonProperty("F0000007")
    private String originatorUserName;

    /**
     * 发起部门
     */
    @JsonProperty("originator_dept_id")
    private String originatorDeptId;

    /**
     * 审批结果: 同意 和 不同意
     */
    @JsonProperty("approvalResult")
    private String approvalResult;

    /**
     * 审批实例业务编号
     */
    @JsonProperty("business_id")
    private String businessId;

    /**
     * 发起部门
     */
    @JsonProperty("F0000002")
    private String originatorDeptName;

    /**
     * 请假事由
     */
    @JsonProperty("F0000005")
    private String leaveType;

    /**
     * 劳动合同所属公司
     */
    @JsonProperty("F0000004")
    private String contractOwnerLtd;

    @JsonProperty("F0000006")
    private String result;

    /**
     * 请假开始时间
     */
    @JsonProperty("F0000016")
    private String startTime;

    /**
     * 请假结束时间
     */
    private String endTime;

    /**
     * 请假时长
     */
    @JsonProperty("F0000017")
    private BigDecimal totalDay;

    /**
     * 请假人Id
     */
    @JsonProperty("F0000018")
    private String askForLeaveUserId;

    /**
     * 根据ProcessInstanceTopVo获得chuanyunAskForLeaveDo
     * @param processInstanceTopVo   传输数据
     * @return  保存数据
     */
    public static ChuanyunAskForLeaveDO fromAskForLeaveDTO(OapiProcessinstanceGetResponse.ProcessInstanceTopVo processInstanceTopVo){
        ChuanyunAskForLeaveDO chuanyunAskForLeaveDo = new ChuanyunAskForLeaveDO();
        chuanyunAskForLeaveDo.setIsCanncel(0);
        if(processInstanceTopVo.getOperationRecords()!= null) {
            if(processInstanceTopVo.getOperationRecords().get(0).getUserid().equals(processInstanceTopVo.getOperationRecords().get(processInstanceTopVo.getOperationRecords().size() -1).getUserid())
                    && processInstanceTopVo.getOperationRecords().get(0).getUserid().equals(processInstanceTopVo.getOriginatorUserid())  ) {
                if(processInstanceTopVo.getOperationRecords().get(0).getOperationType().equals("START_PROCESS_INSTANCE")
                        && processInstanceTopVo.getOperationRecords().get(processInstanceTopVo.getOperationRecords().size() -1).getOperationType().equals("TERMINATE_PROCESS_INSTANCE")) {
                    chuanyunAskForLeaveDo.setIsCanncel(1);
                }
            }
        }
        BeanUtils.copyProperties(processInstanceTopVo, chuanyunAskForLeaveDo);
        List<OapiProcessinstanceGetResponse.FormComponentValueVo> formComponentList = processInstanceTopVo.getFormComponentValues();
        if(formComponentList != null && formComponentList.size() > 0) {
            for (OapiProcessinstanceGetResponse.FormComponentValueVo formComponentValueVo : formComponentList) {
                if ("请假事由".equals(formComponentValueVo.getName())) {
                    chuanyunAskForLeaveDo.setLeaveType(formComponentValueVo.getValue());
                }
                if ("劳动合同所属公司".equals(formComponentValueVo.getName())) {
                    chuanyunAskForLeaveDo.setContractOwnerLtd(formComponentValueVo.getValue());
                }
                if(("[\"开始时间\",\"结束时间\"]").equals(formComponentValueVo.getName())){
                    try {
                        JSONArray json = JSONArray.fromObject(formComponentValueVo.getValue());
                        chuanyunAskForLeaveDo.setStartTime((String)json.get(0));
                        chuanyunAskForLeaveDo.setEndTime((String)json.get(1));
                        chuanyunAskForLeaveDo.setTotalDay(new BigDecimal(String.valueOf(json.get(2))));
                        chuanyunAskForLeaveDo.setTitle((String.valueOf(json.get(0)) + chuanyunAskForLeaveDo.getTitle()));
                    }catch(Exception e) {
                        log.warn("请假信息解析异常");
                        e.printStackTrace();
                    }
                }
            }
        }
        chuanyunAskForLeaveDo.setObjectId(UUID.randomUUID().toString());
        return chuanyunAskForLeaveDo;
    }

}
