package com.galaxis.manatee.entity.chuanyun.data.object;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.galaxis.capsule.bo.ChuanyunObject;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeDeserializer;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.extern.slf4j.Slf4j;

import javax.persistence.*;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/2/13 4:09 下午
 */
@Slf4j
@EqualsAndHashCode(callSuper = true)
@Data
@Entity(name = "CHUANYUN_GROUP_PROJECT")
@JsonIgnoreProperties(ignoreUnknown = true)
public class ChuanyunGroupProjectDO extends BasicDO {

    /**
     * 表名
     */
    public static final String SCHEMA_CODE ="D001789SProjectbase";

    /**
     * 是否已经下发过
     * null or false 未下发过
     * true下发过
     */
    private Boolean isCreate;

    /**
     * 项目编号
     */
    @JsonProperty(value = "F0000002")
    private String projectCode;
    /**
     * 项目名称
     */
    @JsonProperty(value = "F0000003")
    private String projectName;

    /**
     * 外部项目名称
     */
    @JsonProperty(value="F0000038")
    private String outProjectName;

    /**
     * 项目类型
     */
    @JsonProperty(value = "F0000005")
    private String projectType;

    /**
     * 业务集团
     */
    @JsonProperty(value = "F0000033")
    private String businessGroup;
    /**
     * EBG
     */
    public static final String EBG="EBG";
    /**
     * IBG
     */
    public static final String IBG="IBG";

    /**
     * 氚云中项目立项时间
     * U9中项目签订时间、项目启动时间
     */
    @JsonProperty(value = "F0000008")
    @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
    private LocalDateTime projectStartDateTime;

    /**
     * 业务场景
     */
    @JsonProperty(value = "F0000009")
    private String scene;

    /**
     * 项目总监
     * U9项目责任人
     */
    @JsonProperty(value = "F0000006")
    private String projectOwner;

    /**
     * IBG项目经理
     */
    @JsonProperty("F0000012")
    private String ibgProjectManager;

    /**
     * EBG项目经理
     */
    @JsonProperty("F0000011")
    private String ebgProjectManager;

    /**
     * 项目组成员（前端传数据用）
     */
    @Transient
    @JsonProperty("F0000034")
    private String[] projectMemberList;

    /**
     * 项目组成员
     */
    @Column(columnDefinition = "TEXT")
    private String projectMember;

    /**
     * 项目状态
     */
    @JsonProperty("F0000031")
    private String projectStatus;

    /**
     * 签约组织成本中心编码
     */
    @JsonProperty(value = "F0000032")
    private String signCompany;

    /**
     * 关联项目号
     */
    @JsonProperty("F0000040")
    private String relationProjectCode;

    /**
     * 客户编码
     * U9编码
     */
    @JsonProperty(value = "F0000041")
    private String customerCode;

    /**
     * 项目组成员对象
     */
    @Transient
    @JsonProperty("F0000034Object")
    private List<ChuanyunObject> projectMemberObject;

    /**
     * 项目总监数据对象
     */
    @Transient
    @JsonProperty(value = "F0000012Object")
    private ChuanyunObject ibgProjectManagerObject;
    /**
     * 项目总监数据对象
     */
    @Transient
    @JsonProperty(value = "F0000011Object")
    private ChuanyunObject ebgProjectManagerObject;

    /**
     * 项目总监数据对象
     */
    @Transient
    @JsonProperty(value = "F0000006Object")
    private ChuanyunObject projectOwnerObject;

    /**
     * 项目组织信息
     */
    @OneToMany(cascade = CascadeType.ALL, orphanRemoval = true,fetch = FetchType.EAGER)
    @JsonProperty("D001789Fd992bf8fbe114c9babe0f98cf676f8a2")
    private List<ChuanyunGroupProjectDetailDO> detailList;

    //======================================================
    //下面是一开始张曦铭提供的项目的内容，集团总表中缺少这些信息
    //======================================================

//    /**
//     * 下发组织机构编码
//     */
//    @JsonProperty(value = "F0000052")
//    @JsonDeserialize(using = StringArrayToStringDeserializer.class)
//    private String organizationCode;
//
//    /**
//     * 氚云中结项时间
//     * U9中项目结束时间
//     */
//    @JsonProperty(value = "F0000029")
//    @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
//    private LocalDateTime projectEndDateTime;

//

//
//    /**
//     * 项目经理
//     * 多个项目经理用;隔开
//     */
//    @JsonProperty(value = "F0000045")
//    @JsonDeserialize(using = StringArrayToStringDeserializer.class)
//    private String projectManager;
//
//    /**
//     * 项目经理数据对象
//     */
//    @Transient
//    @JsonProperty(value = "F0000045Object")
//    private ChuanyunObject[] projectManagerObject;

}
