package com.galaxis.manatee.entity.chuanyun.data.object;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeDeserializer;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeSerializer;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.extern.slf4j.Slf4j;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.Entity;
import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * @author xuankechen
 */
@Slf4j
@EqualsAndHashCode(callSuper = true)
@Data
@Entity(name = "CHUANYUN_PAYROLL")
@JsonIgnoreProperties(ignoreUnknown = true)
public class ChuanyunPayrollDO extends BasicDO{
    /**
     * 表名
     */
    public static final String SCHEMA_CODE ="D001789Scxgmkkz2hbv7pwy8essawhre4";

    /**
     * 姓名
     */
    @JsonProperty("EmployeeName")
    private String employeeName;

    /**
     * 薪酬期间
     */
    @JsonProperty("Period")
    @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
    @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM")
    private LocalDateTime period;

    /**
     * 工号
     */
    @JsonProperty("ID")
    private String employeeId;

    /**
     * 所属部门
     */
    @JsonProperty("Dept")
    private String dept;

    /**
     * 薪资专员
     */
    @JsonProperty("OwnerId")
    private String ownerId;

    /**
     * 发放状态
     */
    @JsonProperty("SalaryStatus")
    private String salaryStatus;

    /**
     * 基本工资
     */
    @JsonProperty("BasicWage")
    private BigDecimal basicWage;

    /**
     * 应发工资
     */
    @JsonProperty("F0000007")
    private BigDecimal wagesPayable;

    /**
     * 各类津贴
     */
    @JsonProperty("WelfareTotal")
    private BigDecimal welfareTotal;

    /**
     * 处罚扣款
     */
    @JsonProperty("OtherPlus")
    private BigDecimal otherPlus;

    /**
     * 其他扣款
     */
    @JsonProperty("OtherDebit")
    private BigDecimal otherDebit;

    /**
     * 税后其它扣款
     */
    @JsonProperty("F0000015")
    private BigDecimal taxOtherDebit;

    /**
     * 社保代扣
     */
    @JsonProperty("SIDebit")
    private BigDecimal siDebit;

    /**
     * 公积金代扣
     */
    @JsonProperty("HouseDebit")
    private BigDecimal houseDebit;

    /**
     * 公司缴纳社保
     */
    @JsonProperty("CompanySIDebit")
    private BigDecimal companySiDebit;

    /**
     * 公司缴纳公积金
     */
    @JsonProperty("CompanyHouseDebit")
    private BigDecimal companyHouseDebit;

    /**
     * 公司缴纳合计
     */
    @JsonProperty("F0000014")
    private BigDecimal companyTotal;

    /**
     * 所得税
     */
    @JsonProperty("Taxable")
    private BigDecimal taxable;

    /**
     * 实发金额
     */
    @JsonProperty("ActualPay")
    private BigDecimal actualPay;

    /**
     * 绩效奖金包
     */
    @JsonProperty("PerformanceWage")
    private BigDecimal performanceWage;

    /**
     * 个人代扣合计
     */
    @JsonProperty("F0000002")
    private BigDecimal personalTotal;

    /**
     * 公司编码
     */
    @JsonProperty("F0000004")
    private String companyCode;

    /**
     * 加班工资
     */
    @JsonProperty("F0000010")
    private BigDecimal overtimePay;

    /**
     * 考核系数
     */
    @JsonProperty("F0000011")
    private BigDecimal assessment;

    /**
     * 绩效奖金
     */
    @JsonProperty("F0000012")
    private BigDecimal achievementBonus;

    /**
     * 其它应发
     */
    @JsonProperty("F0000013")
    private BigDecimal otherPayables;

    /**
     * 月薪包
     */
    @JsonProperty("F0000016")
    private BigDecimal monthlySalaryPackage;

    /**
     * 岗位津贴
     */
    @JsonProperty("F0000017")
    private BigDecimal postAllowance;

    /**
     * 出勤工资
     */
    @JsonProperty("F0000018")
    private BigDecimal attendanceSalary;

    /**
     * 激励奖金
     */
    @JsonProperty("F0000019")
    private BigDecimal incentiveBonus;

    /**
     * 年终奖计提
     */
    @JsonProperty("F0000021")
    private BigDecimal yearBonusAccrual;

    /**
     * 年终奖发放
     */
    @JsonProperty("F0000020")
    private BigDecimal yearBonusDistribution;

    /**
     * 养老个人
     */
    @JsonProperty("F0000024")
    private BigDecimal personalPension;

    /**
     * 医疗个人
     */
    @JsonProperty("F0000025")
    private BigDecimal personalMedical;

    /**
     * 失业保险
     */
    @JsonProperty("F0000026")
    private BigDecimal unemploymentInsurance;

    /**
     * 养老公司
     */
    @JsonProperty("F0000027")
    private BigDecimal companyPension;

    /**
     * 医疗公司
     */
    @JsonProperty("F0000028")
    private BigDecimal companyMedical;

    /**
     * 失业公司
     */
    @JsonProperty("F0000029")
    private BigDecimal companyUnemployment;

    /**
     * 工伤保险
     */
    @JsonProperty("F0000030")
    private BigDecimal industrialInsurance;

    /**
     * 生育保险
     */
    @JsonProperty("F0000031")
    private BigDecimal maternityInsurance;

    /**
     * 是否外服
     */
    @JsonProperty("F0000032")
    private String isOut;
}
