package com.galaxis.manatee.entity.chuanyun.data.object;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.galaxis.capsule.bo.ChuanyunObject;
import com.galaxis.capsule.dto.ChuanyunProjectDTO;
import com.galaxis.capsule.dto.ChuanyunProjectPrimaryKey;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeDeserializer;
import com.galaxis.manatee.util.StringArrayToStringDeserializer;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

import javax.persistence.Entity;
import javax.persistence.Transient;
import java.time.LocalDateTime;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/2/13 4:09 下午
 */
@Slf4j
@EqualsAndHashCode(callSuper = true)
@Data
@Entity(name = "CHUANYUN_PROJECT")
@JsonIgnoreProperties(ignoreUnknown = true)
public class ChuanyunProjectDO extends BasicDO {

    /**
     * 表名
     */
    public static final String SCHEMA_CODE ="ldx1vawvx19gf8iqxxkusjy05";

    /**
     * 是否已经下发过
     * null or false 未下发过
     * true下发过
     */

    private Boolean isCreate;

    /**
     * 项目编号
     */
    @JsonProperty(value = "F0000002")
    private String projectCode;
    /**
     * 项目名称
     */
    @JsonProperty(value = "F0000001")
    private String projectName;
    /**
     * 项目类型
     */
    @JsonProperty(value = "F0000046")
    private String projectTypeCode;

    /**
     * 设备分类(前端传数据)
     */
    @Transient
    @JsonProperty("F0000056")
    private String[] equipmentTypeList;

    /**
     * 设备分类
     */
    private String equipmentType;

    /**
     * 业务来源
     */
    @JsonProperty("F0000066")
    private String businessSource;

    /**
     * 业务类型
     */
    @JsonProperty("F0000067")
    private String businessType;

    /**
     * 下发组织机构编码
     */
    @JsonProperty(value = "F0000052")
    @JsonDeserialize(using = StringArrayToStringDeserializer.class)
    private String organizationCode;
    /**
     * 氚云中项目立项时间
     * U9中项目签订时间、项目启动时间
     */
    @JsonProperty(value = "F0000028")
    @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
    private LocalDateTime projectStartDateTime;

    /**
     * 氚云中结项时间
     * U9中项目结束时间
     */
    @JsonProperty(value = "F0000029")
    @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
    private LocalDateTime projectEndDateTime;
    /**
     * 客户编码
     * U9编码
     */
    @JsonProperty(value = "F0000054")
    private String customerCode;

    /**
     * 业务场景
     */
    @JsonProperty(value = "F0000055")
    private String scene;

    /**
     * 签约组织成本中心编码
     */
    @JsonProperty(value = "F0000051")
    private String department;

    /**
     * 项目总监
     * U9项目责任人
     */
    @JsonProperty(value = "F0000031")
    private String projectOwner;

    /**
     * 项目总监数据对象
     */
    @Transient
    @JsonProperty(value = "F0000031Object")
    private ChuanyunObject projectOwnerObject;

    /**
     * 项目经理
     * 多个项目经理用;隔开
     */
    @JsonProperty(value = "F0000045")
    @JsonDeserialize(using = StringArrayToStringDeserializer.class)
    private String projectManager;

    /**
     * 项目经理数据对象
     */
    @Transient
    @JsonProperty(value = "F0000045Object")
    private ChuanyunObject[] projectManagerObject;

    /**
     * 拥有者
     */
    @JsonProperty("OwnerId")
    private String ownerId;

    /**
     * 拥有者对象
     */
    @Transient
    @JsonProperty("OwnerIdObject")
    private ChuanyunObject ownerIdObject;

    public static ChuanyunProjectDTO toChuanyunProjectDTO(ChuanyunProjectDO chuanyunProjectDO){
        ChuanyunProjectDTO chuanyunProjectDTO=new ChuanyunProjectDTO();
        //制作联合主键
        ChuanyunProjectPrimaryKey chuanyunProjectPrimaryKey=new ChuanyunProjectPrimaryKey();
        chuanyunProjectPrimaryKey.setProjectCode(chuanyunProjectDO.getProjectCode());
        chuanyunProjectPrimaryKey.setOwnerOrganizationCode(chuanyunProjectDO.getOrganizationCode());
        chuanyunProjectDTO.setChuanyunProjectPrimaryKey(chuanyunProjectPrimaryKey);
        chuanyunProjectDTO.setProjectName(chuanyunProjectDO.getProjectName());
        try{
            chuanyunProjectDTO.setProjectTypeCode(ProjectType.fromTypeName(chuanyunProjectDO.getProjectTypeCode()).typeCode);
        }catch (Exception e){
            log.warn(e.getMessage());
        }
        chuanyunProjectDTO.setProjectSignDateTime(chuanyunProjectDO.getProjectStartDateTime());
        chuanyunProjectDTO.setCustomerCode(chuanyunProjectDO.getCustomerCode());
        chuanyunProjectDTO.setScene(chuanyunProjectDO.getScene());
        chuanyunProjectDTO.setDepartment(chuanyunProjectDO.getDepartment());
        chuanyunProjectDTO.setPlanStartDate(chuanyunProjectDO.getProjectStartDateTime());
        chuanyunProjectDTO.setPlanEndDate(chuanyunProjectDO.getProjectEndDateTime());
        chuanyunProjectDTO.setProjectManager(ChuanYunConstant.USER_EMPLOYEE_NUMBER.get(chuanyunProjectDO.getProjectManager()));
        chuanyunProjectDTO.setProjectOwner(ChuanYunConstant.USER_EMPLOYEE_NUMBER.get(chuanyunProjectDO.getProjectOwner()));
        return chuanyunProjectDTO;
    }

    /**
     * 项目类型映射枚举
     */
    @Getter
    enum ProjectType{
        /**
         * 设备相关项目
         */
        EQUIPMENT("纯设备","10"),
        /**
         * 硬集成项目
         */
        EQUIPMENT_INTEGRATION("硬集成（含设备）","20"),
        /**
         * 软集成项目
         */
        SOFTWARE_INTEGRATION("软集成（不含设备）","30"),
        /**
         * 纯软件
         */
        SOFTWARE("纯软件","40"),
        /**
         * 售后
         */
        MAINTENANCE("售后项目","50"),
        /**
         * 规划咨询
         */
        CONSULTATION("纯规划咨询","60"),
        /**
         * 其他
         */
        OTHER("其他","100");
        private final String typeName;
        private final String typeCode;

        ProjectType(String typeName, String typeCode) {
            this.typeName = typeName;
            this.typeCode = typeCode;
        }

        /**
         * 根据类型名称获取类型枚举
         * @param typeName  类型名称
         * @return  枚举
         */
        static ProjectType fromTypeName(String typeName) throws Exception {
            for (ProjectType projectType:ProjectType.values()){
                if (projectType.typeName.equals(typeName)){
                    return projectType;
                }
            }
            throw new Exception("peojectType枚举类型不匹配");
        }


    }
}
