package com.galaxis.manatee.manager;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.galaxis.manatee.entity.chuanyun.dto.*;
import com.galaxis.manatee.exception.BigSizeException;

import java.util.List;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/1/9 11:57 下午
 */
public interface ChuanYunManager {

    /**
     * 查询单条数据
     *
     * @param schemaCode  表单编码
     * @param bizObjectId 数据Id
     * @param <T>         查询数据类型
     * @return 单条数据返回结果
     */
    <T> ChuanyunFindBizDTO<T> find(String schemaCode, String bizObjectId);

    /**
     * 自定义数据查询接口
     * @param appCode       appCode自定义接口所在应用的应用编码，
     *                      查看应用编码的方法：列表页面点击左上角应用名后的设置按钮，
     *                      再点击重命名，在弹窗中可以看到当前应用的应用编码
     * @param actionName    用来给自定义接口判断本次请求的动作
     * @param controller    是自定义接口的类名
     * @return              氚云返回数据
     */
    ChuanyunFindDTO find(String appCode, String actionName, String controller);

    /**
     * 根据filter批量查询
     *
     * @param schemaCode 表单编码
     * @param filter     查询条件
     * @param <T>        查询数据类型
     * @throws JsonProcessingException 抛出异常
     * @return 批量返回结果
     */
    <T> ChuanyunFindAllBizDTO<T> findAll(String schemaCode, Filter filter) throws JsonProcessingException;

    /**
     * 自定义数据查询接口
     * @param appCode       appCode自定义接口所在应用的应用编码，
     *                      查看应用编码的方法：列表页面点击左上角应用名后的设置按钮，
     *                      再点击重命名，在弹窗中可以看到当前应用的应用编码
     * @param actionName    用来给自定义接口判断本次请求的动作
     * @param controller    是自定义接口的类名
     * @return              氚云返回数据
     */
    ChuanyunFindAllDTO findAll(String appCode, String actionName, String controller);

    /**
     * 保存
     *
     * @param isSubmit   是否为提交状态
     * @param schemaCode 表单编码
     * @param bizObject  数据字符串
     * @throws BigSizeException 保存数据超长的异常
     * @return 保存结果
     */
    ChuanyunSaveDTO save(String schemaCode, String bizObject, Boolean isSubmit) throws BigSizeException;

    /**
     * 批量保存
     * @param isSubmit       是否直接提交
     * @param schemaCode     表单编码
     * @param bizObjectArray 数据字符串
     * @throws BigSizeException 保存数据超长的异常
     * @throws JsonProcessingException  json转化异常
     * @return 保存结果
     */
    ChuanyunSaveAllDTO saveAll(String schemaCode, List<String> bizObjectArray, Boolean isSubmit) throws JsonProcessingException, BigSizeException;

    /**
     * 保存
     *
     * @param schemaCode 表单编码
     * @param bizObjectId   主键
     * @param bizObject  数据字符串
     * @throws BigSizeException 保存数据超长的异常
     * @return 保存结果
     */
    ChuanyunSaveDTO update(String schemaCode,String bizObjectId, String bizObject) throws BigSizeException;

    /**
     * 删除
     * @param schemaCode
     * @param bizObjectId
     * @return
     */
    ChuanyunSaveDTO delete(String schemaCode, String bizObjectId);
}
