package com.galaxis.manatee.manager.impl;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.entity.chuanyun.dto.*;
import com.galaxis.manatee.exception.BigSizeException;
import com.galaxis.manatee.manager.ChuanYunManager;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.List;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/2/5 4:02 下午
 */
@Slf4j
@Service
public class DefaultChuanyunManagerImpl implements ChuanYunManager {
    /**
     * 字符串最大长度
     */
    private final Integer maxLength=1048576;

    @Override
    public ChuanyunFindDTO find(String appCode, String actionName, String controller) {
        @Data
        @AllArgsConstructor
        class Find{
            @JsonProperty(value = "ActionName")
            private String actionName;
            @JsonProperty(value = "Controller")
            private String controller;
            @JsonProperty(value = "AppCode")
            private String appCode;
        }
        Find find=new Find(actionName,controller,appCode);
        //请求客户端
        RestTemplate restTemplate=new RestTemplate();
        HttpHeaders httpHeaders=new HttpHeaders();
        httpHeaders.add("EngineCode",ChuanYunConstant.ENGINE_CODE);
        httpHeaders.add("EngineSecret",ChuanYunConstant.ENGINE_SECRET);
        HttpEntity<Find> httpEntity=new HttpEntity<>(find,httpHeaders);
        ChuanyunFindDTO chuanyunFindAllResponse =new ChuanyunFindDTO();
        return restTemplate.postForObject(ChuanYunConstant.CHUAN_YUN_INVOKE_URL,httpEntity,chuanyunFindAllResponse.getClass() );
    }

    @Override
    public synchronized ChuanyunFindAllDTO findAll(String appCode, String actionName, String controller) {
        @Data
        @AllArgsConstructor
        class FindAll{
            @JsonProperty(value = "ActionName")
            private String actionName;
            @JsonProperty(value = "Controller")
            private String controller;
            @JsonProperty(value = "AppCode")
            private String appCode;
        }
        FindAll findAll=new FindAll(actionName,controller,appCode);
        //请求客户端
        RestTemplate restTemplate=new RestTemplate();
        HttpHeaders httpHeaders=new HttpHeaders();
        httpHeaders.add("EngineCode",ChuanYunConstant.ENGINE_CODE);
        httpHeaders.add("EngineSecret",ChuanYunConstant.ENGINE_SECRET);
        HttpEntity<FindAll> httpEntity=new HttpEntity<>(findAll,httpHeaders);
        ChuanyunFindAllDTO chuanyunFindAllResponse =new ChuanyunFindAllDTO();
        return restTemplate.postForObject(ChuanYunConstant.CHUAN_YUN_INVOKE_URL,httpEntity,chuanyunFindAllResponse.getClass() );
    }

    @Override
    public <T> ChuanyunFindBizDTO<T> find(String schemaCode, String bizObjectId) {
        @Data
        class Find{
            @JsonProperty(value = "ActionName")
            private String actionName="LoadBizObject";
            @JsonProperty(value = "SchemaCode")
            private String schemaCode;
            @JsonProperty(value = "BizObjectId")
            private String bizObjectId;

            public Find(String schemaCode, String bizObjectId) {
                this.schemaCode = schemaCode;
                this.bizObjectId = bizObjectId;
            }
        }
        Find find=new Find(schemaCode,bizObjectId);
        //请求客户端
        RestTemplate restTemplate=new RestTemplate();
        HttpHeaders httpHeaders=new HttpHeaders();
        httpHeaders.add("EngineCode",ChuanYunConstant.ENGINE_CODE);
        httpHeaders.add("EngineSecret",ChuanYunConstant.ENGINE_SECRET);
        HttpEntity<Find> httpEntity=new HttpEntity<>(find,httpHeaders);
        ChuanyunFindBizDTO<T> chuanyunFindResponse =new ChuanyunFindBizDTO<>();
        return restTemplate.postForObject(ChuanYunConstant.CHUAN_YUN_INVOKE_URL,httpEntity,chuanyunFindResponse.getClass() );
    }

    @Override
    public <T> ChuanyunFindAllBizDTO<T> findAll(String schemaCode, Filter filter) throws JsonProcessingException {
        @Data
        class FindList{
            @JsonProperty(value = "ActionName")
            private String actionName="LoadBizObjects";
            @JsonProperty(value = "SchemaCode")
            private String schemaCode;
            @JsonProperty(value = "Filter")
            private String filter;

            public FindList(String schemaCode, String filter) {
                this.schemaCode = schemaCode;
                this.filter = filter;
            }
        }
        ObjectMapper objectMapper=new ObjectMapper();
        //构建请求体
        FindList findList=new FindList(schemaCode,objectMapper.writeValueAsString(filter));
        //请求客户端
        RestTemplate restTemplate=new RestTemplate();
        HttpHeaders httpHeaders=new HttpHeaders();
        httpHeaders.add("EngineCode",ChuanYunConstant.ENGINE_CODE);
        httpHeaders.add("EngineSecret",ChuanYunConstant.ENGINE_SECRET);
        HttpEntity<FindList> httpEntity=new HttpEntity<>(findList,httpHeaders);
        ChuanyunFindAllBizDTO<T> chuanyunFindAllResponse =new ChuanyunFindAllBizDTO<>();
        return restTemplate.postForObject(ChuanYunConstant.CHUAN_YUN_INVOKE_URL,httpEntity, chuanyunFindAllResponse.getClass());
    }

    @Override
    public ChuanyunSaveDTO save(String schemaCode, String bizObject, Boolean isSubmit) throws BigSizeException {
        @Data
        class Save{
            @JsonProperty(value = "ActionName")
            private String actionName="CreateBizObject";
            @JsonProperty(value = "SchemaCode")
            private String schemaCode;
            @JsonProperty(value = "BizObject")
            private String bizObject;
            @JsonProperty(value = "IsSubmit")
            private Boolean isSubmit;

            public Save(String schemaCode, String bizObject, Boolean isSubmit) {
                this.schemaCode = schemaCode;
                this.bizObject = bizObject;
                this.isSubmit = isSubmit;
            }
        }
        //如果数据量太大就抛出异常
        if(bizObject.length()>maxLength){
            throw new BigSizeException(bizObject);
        }
        Save save=new Save(schemaCode,bizObject,isSubmit);
        //请求客户端
        RestTemplate restTemplate=new RestTemplate();
        HttpHeaders httpHeaders=new HttpHeaders();
        httpHeaders.add("EngineCode",ChuanYunConstant.ENGINE_CODE);
        httpHeaders.add("EngineSecret",ChuanYunConstant.ENGINE_SECRET);
        HttpEntity<Save> httpEntity=new HttpEntity<>(save,httpHeaders);
        ChuanyunSaveDTO chuanyunSaveResponse =new ChuanyunSaveDTO();
        chuanyunSaveResponse = restTemplate.postForObject(ChuanYunConstant.CHUAN_YUN_INVOKE_URL,httpEntity, chuanyunSaveResponse.getClass());
        if (chuanyunSaveResponse!=null&&chuanyunSaveResponse.getSuccessful()) {
//            log.info("新增成功！");
        }else {
            log.info("新增失败！");
        }
        return chuanyunSaveResponse;
    }

    @Override
    public ChuanyunSaveAllDTO saveAll(String schemaCode, List<String> bizObjectArray, Boolean isSubmit) throws JsonProcessingException, BigSizeException {
        @Data
        class SaveAll{
            @JsonProperty(value = "ActionName")
            private String actionName="CreateBizObjects";
            @JsonProperty(value = "SchemaCode")
            private String schemaCode;
            @JsonProperty(value = "BizObjectArray")
            private List<String> bizObjectArray;
            @JsonProperty(value = "IsSubmit")
            private Boolean isSubmit;

            public SaveAll(String schemaCode, List<String> bizObjectArray, Boolean isSubmit) {
                this.schemaCode = schemaCode;
                this.bizObjectArray = bizObjectArray;
                this.isSubmit = isSubmit;
            }
        }
        var objectMapper=new ObjectMapper();
        //如果数据量太大就抛出异常
        if(objectMapper.writeValueAsString(bizObjectArray).length()>maxLength){
            throw new BigSizeException(bizObjectArray+"");
        }
        SaveAll saveAll=new SaveAll(schemaCode,bizObjectArray,isSubmit);
        //请求客户端
        RestTemplate restTemplate=new RestTemplate();
        HttpHeaders httpHeaders=new HttpHeaders();
        httpHeaders.add("EngineCode",ChuanYunConstant.ENGINE_CODE);
        httpHeaders.add("EngineSecret",ChuanYunConstant.ENGINE_SECRET);
        HttpEntity<SaveAll> httpEntity=new HttpEntity<>(saveAll,httpHeaders);
        ChuanyunSaveAllDTO chuanyunSaveAllResponse =new ChuanyunSaveAllDTO();
        chuanyunSaveAllResponse = restTemplate.postForObject(ChuanYunConstant.CHUAN_YUN_INVOKE_URL,httpEntity, chuanyunSaveAllResponse.getClass());
        if (chuanyunSaveAllResponse!=null&&chuanyunSaveAllResponse.getSuccessful()) {
//            log.info("批量新增成功！");
        }else {
            log.info("批量新增失败！");
        }
        return chuanyunSaveAllResponse;
    }

    @Override
    public ChuanyunSaveDTO update(String schemaCode, String bizObjectId, String bizObject) throws BigSizeException {
        @Data
        class Update{
            @JsonProperty(value = "ActionName")
            private String actionName="UpdateBizObject";
            @JsonProperty(value = "SchemaCode")
            private String schemaCode;
            @JsonProperty(value = "BizObject")
            private String bizObject;
            @JsonProperty(value = "BizObjectId")
            private String bizObjectId;

            public Update(String schemaCode, String bizObject,String bizObjectId) {
                this.schemaCode = schemaCode;
                this.bizObject = bizObject;
                this.bizObjectId=bizObjectId;
            }
        }
        //如果数据量太大就抛出异常
        if(bizObject.length()>maxLength){
            throw new BigSizeException(bizObject);
        }
        Update update=new Update(schemaCode,bizObject,bizObjectId);
        //请求客户端
        RestTemplate restTemplate=new RestTemplate();
        HttpHeaders httpHeaders=new HttpHeaders();
        httpHeaders.add("EngineCode",ChuanYunConstant.ENGINE_CODE);
        httpHeaders.add("EngineSecret",ChuanYunConstant.ENGINE_SECRET);
        HttpEntity<Update> httpEntity=new HttpEntity<>(update,httpHeaders);
        ChuanyunSaveDTO chuanyunSaveResponse =new ChuanyunSaveDTO();
        chuanyunSaveResponse = restTemplate.postForObject(ChuanYunConstant.CHUAN_YUN_INVOKE_URL,httpEntity, chuanyunSaveResponse.getClass());
        if (chuanyunSaveResponse!=null&&chuanyunSaveResponse.getSuccessful()) {
//            log.info(bizObjectId+"更新成功！");
        }else {
            log.info(bizObjectId+"更新失败！"+bizObject);
        }
        return chuanyunSaveResponse;
    }

    @Override
    public ChuanyunSaveDTO delete(String schemaCode, String bizObjectId) {
        @Data
        class Find{
            @JsonProperty(value = "ActionName")
            private String actionName="RemoveBizObject";
            @JsonProperty(value = "SchemaCode")
            private String schemaCode;
            @JsonProperty(value = "BizObjectId")
            private String bizObjectId;

            public Find(String schemaCode, String bizObjectId) {
                this.schemaCode = schemaCode;
                this.bizObjectId = bizObjectId;
            }
        }
        Find find=new Find(schemaCode,bizObjectId);
        //请求客户端
        RestTemplate restTemplate=new RestTemplate();
        HttpHeaders httpHeaders=new HttpHeaders();
        httpHeaders.add("EngineCode",ChuanYunConstant.ENGINE_CODE);
        httpHeaders.add("EngineSecret",ChuanYunConstant.ENGINE_SECRET);
        HttpEntity<Find> httpEntity=new HttpEntity<>(find,httpHeaders);
        ChuanyunSaveDTO chuanyunSaveResponse =new ChuanyunSaveDTO();
        return chuanyunSaveResponse = restTemplate.postForObject(ChuanYunConstant.CHUAN_YUN_INVOKE_URL,httpEntity, chuanyunSaveResponse.getClass());
    }
}
