package com.galaxis.manatee.service;

import com.dingtalk.api.DingTalkClient;
import com.dingtalk.api.request.*;
import com.dingtalk.api.response.OapiProcessinstanceGetResponse;
import com.dingtalk.api.response.OapiProcessinstanceListidsResponse;
import com.galaxis.manatee.constant.DingTalkConstant;
import com.galaxis.manatee.dao.ChuanyunAskForLeaveDao;
import com.galaxis.manatee.dao.DingTalkProcessInstanceDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunAskForLeaveDO;
import com.galaxis.manatee.entity.ding.DingTalkProcessInstance;
import com.taobao.api.ApiException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.temporal.ChronoUnit;
import java.util.*;

/**
 * 钉钉定时任务
 * @author zcj
 * @version 0.1
 * @date 2020/1/10 2:28 下午
 */
@Slf4j
@Service
public class DingTalkScheduledTask {

    /**
     * 审批Id列表客户端
     */
    private final DingTalkClient processInstanceListIdsClient;

    /**
     * 审批实例客户端
     */
    private final DingTalkClient processInstanceClient;

    /**
     * 钉钉审批实例保存方法
     */
    private final DingTalkProcessInstanceDao dingTalkProcessInstanceDao;

    /**
     * 钉钉请假数据同步到氚云
     */
    private final ChuanyunAskForLeaveDao chuanyunAskForLeaveDao;

    public DingTalkScheduledTask(DingTalkClient processInstanceListIdsClient, DingTalkClient processInstanceClient,
                                 ChuanyunAskForLeaveDao chuanyunAskForLeaveDao,DingTalkProcessInstanceDao dingTalkProcessInstanceDao) {
        this.processInstanceListIdsClient = processInstanceListIdsClient;
        this.processInstanceClient = processInstanceClient;
        this.dingTalkProcessInstanceDao = dingTalkProcessInstanceDao;
        this.chuanyunAskForLeaveDao = chuanyunAskForLeaveDao;
    }

    /**
     * 每个小时定期更新出差申请数据
     * 出差申请信息暂时不进行更新
     */
    @Scheduled(fixedDelay = 3600000L,initialDelay = 10000L)
    void updateBusinessTripProcessInstanceIdList() throws ApiException {
        updateProcessInstanceIdList(DingTalkConstant.PROCESS_CODE_BUSINESS_TRIP);
    }



    /**
     * 每小时根据出差申请Id跟新出差申请信息。
     * 出差申请信息暂时不进行更新
     */
    @Scheduled(fixedDelay = 3600000L,initialDelay = 10000L)
    void updateProcessInstanceList() throws ApiException {
         updateChuanyunAskForLeave(DingTalkConstant.PROCESS_CODE_ASKFORLEAVE);
    }

    private void updateChuanyunAskForLeave(String businessTripProcessCode) throws ApiException {
            var flag=true;
            //单个请求
            var oApiProcessInstanceListIdsRequest=new OapiProcessinstanceListidsRequest();
            oApiProcessInstanceListIdsRequest.setProcessCode(businessTripProcessCode);
            oApiProcessInstanceListIdsRequest.setStartTime(LocalDateTime.of(2019,1,1,0,0).toInstant(ZoneOffset.UTC).toEpochMilli());
            oApiProcessInstanceListIdsRequest.setEndTime(LocalDateTime.now().toInstant(ZoneOffset.UTC).toEpochMilli());
            oApiProcessInstanceListIdsRequest.setCursor(0L);
            oApiProcessInstanceListIdsRequest.setSize(20L);
            while(flag){
                LocalDateTime start=LocalDateTime.now();
                OapiProcessinstanceListidsResponse oapiProcessinstanceListidsResponse=processInstanceListIdsClient.execute(oApiProcessInstanceListIdsRequest,DingTalkConstant.DING_TALK_TOKEN);
                List<ChuanyunAskForLeaveDO> chuanyunAskForLeaveDoList=new ArrayList<>();
                if(oapiProcessinstanceListidsResponse.getErrcode()!=0){
                    log.info(oapiProcessinstanceListidsResponse.getErrmsg());
                    try {
                        Thread.sleep(5000);
                    } catch (InterruptedException e) {
                        log.error(e.getMessage());
                    }
                    continue;
                }
                //逐个请求
                oapiProcessinstanceListidsResponse.getResult().getList().forEach(resultId->{
                    Optional<DingTalkProcessInstance> instance=dingTalkProcessInstanceDao.findById(resultId);
                    if(instance.isEmpty()){
                        OapiProcessinstanceGetRequest request=new OapiProcessinstanceGetRequest();
                        request.setProcessInstanceId(resultId);
                        try {
                            OapiProcessinstanceGetResponse oapiProcessinstanceGetResponse = processInstanceClient.execute(request, DingTalkConstant.DING_TALK_TOKEN);
                            ChuanyunAskForLeaveDO chuanyunAskForLeaveDo = ChuanyunAskForLeaveDO.fromAskForLeaveDTO(oapiProcessinstanceGetResponse.getProcessInstance());
                            List<ChuanyunAskForLeaveDO>   existsAskForLeave = chuanyunAskForLeaveDao.findByTitleSQL(chuanyunAskForLeaveDo.getTitle());
                            if(existsAskForLeave != null && !existsAskForLeave.isEmpty()) {
                                chuanyunAskForLeaveDao.deleteById(existsAskForLeave.get(0).getObjectId());
                            }
                            chuanyunAskForLeaveDoList.add(chuanyunAskForLeaveDo);
                        } catch (ApiException e) {
                            e.printStackTrace();
                        }
                    }
                });

                //保存id列表和实例列表,如果有更新的内容
                chuanyunAskForLeaveDao.saveAll(chuanyunAskForLeaveDoList);
                log.info(businessTripProcessCode+"下一个cursor为"+oapiProcessinstanceListidsResponse.getResult().getNextCursor()+"保存成功"+"耗时"+ ChronoUnit.SECONDS.between(start,LocalDateTime.now())+"秒");
                flag=null!=oapiProcessinstanceListidsResponse.getResult().getNextCursor();
                if (flag){
                    oApiProcessInstanceListIdsRequest.setCursor(oapiProcessinstanceListidsResponse.getResult().getNextCursor());
                }
            }
    }

    private void updateProcessInstanceIdList(String businessTripProcessCode) throws ApiException {
        var flag=true;
        //单个请求
        var oApiProcessInstanceListIdsRequest=new OapiProcessinstanceListidsRequest();
        oApiProcessInstanceListIdsRequest.setProcessCode(businessTripProcessCode);
        oApiProcessInstanceListIdsRequest.setStartTime(LocalDateTime.of(2019,1,1,0,0).toInstant(ZoneOffset.UTC).toEpochMilli());
        oApiProcessInstanceListIdsRequest.setEndTime(LocalDateTime.now().toInstant(ZoneOffset.UTC).toEpochMilli());
        oApiProcessInstanceListIdsRequest.setCursor(0L);
        oApiProcessInstanceListIdsRequest.setSize(20L);
        while(flag){
            LocalDateTime start=LocalDateTime.now();
            OapiProcessinstanceListidsResponse oapiProcessinstanceListidsResponse=processInstanceListIdsClient.execute(oApiProcessInstanceListIdsRequest,DingTalkConstant.DING_TALK_TOKEN);
            List<DingTalkProcessInstance> dingTalkProcessInstanceList=new ArrayList<>();
            if(oapiProcessinstanceListidsResponse.getErrcode()!=0){
                log.info(oapiProcessinstanceListidsResponse.getErrmsg());
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    log.error(e.getMessage());
                }
                continue;
            }
            //逐个请求
            oapiProcessinstanceListidsResponse.getResult().getList().forEach(resultId->{
                Optional<DingTalkProcessInstance> instance=dingTalkProcessInstanceDao.findById(resultId);
                if(instance.isEmpty()){
                    //&&instance.get()==null
                    //淘宝SDK方法
                    OapiProcessinstanceGetRequest request=new OapiProcessinstanceGetRequest();
                    request.setProcessInstanceId(resultId);
                    DingTalkProcessInstance dingTalkProcessInstance=new DingTalkProcessInstance(resultId,businessTripProcessCode);
                    try {
                        OapiProcessinstanceGetResponse oapiProcessinstanceGetResponse = processInstanceClient.execute(request, DingTalkConstant.DING_TALK_TOKEN);
                        dingTalkProcessInstance.setProcessInstance(oapiProcessinstanceGetResponse.getProcessInstance());
                    } catch (ApiException e) {
                        e.printStackTrace();
                    }
                    //淘宝SDK方法结束
                    dingTalkProcessInstanceList.add(dingTalkProcessInstance);
                }
            });

            //保存id列表和实例列表
            dingTalkProcessInstanceDao.saveAll(dingTalkProcessInstanceList);
            log.info(businessTripProcessCode+"下一个cursor为"+oapiProcessinstanceListidsResponse.getResult().getNextCursor()+"保存成功"+"耗时"+ ChronoUnit.SECONDS.between(start,LocalDateTime.now())+"秒");
            flag=null!=oapiProcessinstanceListidsResponse.getResult().getNextCursor();
            if (flag){
                oApiProcessInstanceListIdsRequest.setCursor(oapiProcessinstanceListidsResponse.getResult().getNextCursor());
            }
        }
    }
}
