package com.galaxis.manatee.task;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.capsule.dto.PageDTO;
import com.galaxis.manatee.dao.*;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunEquipmentDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunEquipmentInformDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunGroupProjectDO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunDeviceConfigureDTO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunDeviceDTO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSaveDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.exception.BigSizeException;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.manager.ClawFeign;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@Service
@Slf4j
public class EquipmentInfoTask {

    private final Integer pageSize=20;
    private final ChuanYunManager chuanYunManager;
    private final ClawFeign clawFeign;
    private final ChuanyunMaterialDao chuanyunMaterialDao;


    public EquipmentInfoTask(ChuanYunManager chuanYunManager, ClawFeign clawFeign, ChuanyunMaterialDao chuanyunMaterialDao) {
        this.chuanYunManager = chuanYunManager;
        this.clawFeign = clawFeign;
        this.chuanyunMaterialDao = chuanyunMaterialDao;
    }

    @Scheduled(fixedDelay = 3600000L)
    private void getDataFromChuanyun() {
        saveDeviceInfo();
        saveDeviceConfigure();
    }

    /**
     * U9设备信息
     */
    private void saveDeviceInfo(){
        var objectMapper=new ObjectMapper();
        var totalCount=0L;
        var page = 1;
        var size = 20;
        var flag=true;
        while (flag){
            Map<String,String> map=new HashMap<>(1);
            map.put("page",String.valueOf(page));
            map.put("size",String.valueOf(size));
            PageDTO<ChuanyunDeviceDTO> pageInfo=clawFeign.findChuanyunDevice(map);
            totalCount=pageInfo.getTotalElements();
            if (page<=pageInfo.getTotalPages()){
                page += 1;
            }else {
                flag = false;
                break;
            }
            List<ChuanyunDeviceDTO> list = pageInfo.getContent();
            list.forEach(chuanyunDeviceDTO -> {
                try {
                    if(chuanyunDeviceDTO != null) {
                        List<String> projectmatchers = new ArrayList<>();
                        projectmatchers.add("F0000002_2,"+chuanyunDeviceDTO.getProjectcode());
                        var projectfilter= Filter.instance(0,Integer.MAX_VALUE,true,"And",projectmatchers);
                        var chuanyunFindResponse = chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE,projectfilter);
                        if (chuanyunFindResponse.getReturnData()!=null){
                            List<ChuanyunGroupProjectDO> projectresult=objectMapper.convertValue(chuanyunFindResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                            projectresult.forEach(chuanyunGroupProjectDO -> {
                                if (chuanyunGroupProjectDO.getProjectType().equals("实施")||chuanyunGroupProjectDO.getProjectType().equals("售后")){
                                    chuanyunDeviceDTO.setProjectcode(chuanyunGroupProjectDO.getObjectId());
                                }
                            });
                        }

                        chuanyunMaterialDao.findByCode(chuanyunDeviceDTO.getItemcode()).ifPresent(chuanyunMaterialDO -> {
                            chuanyunDeviceDTO.setItemcode(chuanyunMaterialDO.getObjectId());
                        });
                        DeviceInfoDTO deviceInfoDTO = new DeviceInfoDTO(chuanyunDeviceDTO);
                        var purcostString=objectMapper.writeValueAsString(deviceInfoDTO);
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        //根据项目号判断是否存在
                        List<String> macthers = new ArrayList<>();
                        macthers.add("F0000001_2,"+chuanyunDeviceDTO.getSncode());
                        var filter= Filter.instance(0,1,true,"And",macthers);
                        var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunEquipmentInformDO.SCHEMA_CODE,filter);
                        if (chuanyunFindAllResponse.getReturnData() == null) {
                            //新增
                            chuanyunSaveDTO =chuanYunManager.save(ChuanyunEquipmentInformDO.SCHEMA_CODE,purcostString,true);
                        }else {
                            List<ChuanyunEquipmentInformDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                            ChuanyunEquipmentInformDO chuanyunProjectCashFlowDO = result.get(0);
                            //更新
                            chuanyunSaveDTO = chuanYunManager.update(ChuanyunEquipmentInformDO.SCHEMA_CODE,chuanyunProjectCashFlowDO.getObjectId(),purcostString);
                        }
                        if(chuanyunSaveDTO.getSuccessful()){
//                            log.info("新增设备信息成功");
                        }else{
                            log.warn("新增设备信息失败");
                        }
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            });
        }
        log.info("设备信息完成更更新"+totalCount);
    }

    /**
     * U9获取设备配置信息
     */
    private void saveDeviceConfigure(){
        var objectMapper=new ObjectMapper();
        var totalCount=0L;
        var page = 1;
        var size = 20;
        var flag=true;
        while (flag){
            Map<String,String> map=new HashMap<>(1);
            map.put("page",String.valueOf(page));
            map.put("size",String.valueOf(size));
            PageDTO<ChuanyunDeviceConfigureDTO> pageInfo=clawFeign.findChuanyunDeviceConfigure(map);
            totalCount=pageInfo.getTotalElements();
            if (page<=pageInfo.getTotalPages()){
                page += 1;
            }else {
                flag = false;
                break;
            }
            List<ChuanyunDeviceConfigureDTO> list = pageInfo.getContent();
            list.forEach(chuanyunDeviceConfigureDTO -> {
                try {
                    List<String> matchers = new ArrayList<>();
                    matchers.add("F0000001_2,"+chuanyunDeviceConfigureDTO.getChuanyunDeviceConfigurePrimaryKey().getSncode());
                    var filter= Filter.instance(0,1,true,"And",matchers);
                    var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunEquipmentInformDO.SCHEMA_CODE,filter);
                    if (chuanyunFindAllResponse.getReturnData()!=null){
                        List<ChuanyunEquipmentInformDO> projectresult=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                        chuanyunDeviceConfigureDTO.getChuanyunDeviceConfigurePrimaryKey().setSncode(projectresult.get(0).getObjectId());
                    }

                    List<String> projectmatchers = new ArrayList<>();
                    projectmatchers.add("F0000001_2,"+chuanyunDeviceConfigureDTO.getChuanyunDeviceConfigurePrimaryKey().getSncode());
                    projectmatchers.add("F0000002_2,"+chuanyunDeviceConfigureDTO.getChuanyunDeviceConfigurePrimaryKey().getLinenum());
                    var projectfilter= Filter.instance(0,Integer.MAX_VALUE,true,"And",projectmatchers);
                    var chuanyunFindResponse = chuanYunManager.findAll(ChuanyunEquipmentDO.SCHEMA_CODE,projectfilter);
                    ChuanyunSaveDTO chuanyunSaveDTO;
                    EquipmentDTO equipmentDTO = new EquipmentDTO(chuanyunDeviceConfigureDTO);
                    var purcostString=objectMapper.writeValueAsString(equipmentDTO);
                    if (chuanyunFindResponse.getReturnData()!=null){
                        List<ChuanyunEquipmentDO> projectresult=objectMapper.convertValue(chuanyunFindResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                        chuanyunSaveDTO = chuanYunManager.update(ChuanyunEquipmentDO.SCHEMA_CODE,projectresult.get(0).getObjectId(),purcostString);
                    }else {
                        chuanyunSaveDTO =chuanYunManager.save(ChuanyunEquipmentDO.SCHEMA_CODE,purcostString,true);
                    }
                    if(chuanyunSaveDTO.getSuccessful()){
//                        log.info("新增配置设备信息成功");
                    }else{
                        log.warn("新增配置设备信息失败");
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            });
        }
        log.info("设备配置信息完成更新"+totalCount);
    }

    @Data
    class EquipmentDTO{
        /**
         * 设备编号
         */
        @JsonProperty("F0000001")
        private String equipmentCode;

        /**
         * 设备配置编号
         */
        @JsonProperty("F0000002")
        private String equipmentConfigureCode;

        /**
         * 设备配置属性
         */
        @JsonProperty("F0000003")
        private String equipmentConfigureAttribute;

        /**
         * 设备配置值
         */
        @JsonProperty("F0000004")
        private String equipmentConfigureValue;

        public EquipmentDTO(ChuanyunDeviceConfigureDTO chuanyunDeviceConfigureDTO){
            if (null!=chuanyunDeviceConfigureDTO){
                if (null!=chuanyunDeviceConfigureDTO.getChuanyunDeviceConfigurePrimaryKey().getSncode()){
                    this.equipmentCode=chuanyunDeviceConfigureDTO.getChuanyunDeviceConfigurePrimaryKey().getSncode();
                }
                if (null!=chuanyunDeviceConfigureDTO.getChuanyunDeviceConfigurePrimaryKey().getLinenum()){
                    this.equipmentConfigureCode=chuanyunDeviceConfigureDTO.getChuanyunDeviceConfigurePrimaryKey().getLinenum();
                }
                if (null!=chuanyunDeviceConfigureDTO.getPropertyname()){
                    this.equipmentConfigureAttribute=chuanyunDeviceConfigureDTO.getPropertyname();
                }
                if (null!=chuanyunDeviceConfigureDTO.getPropertyvaluename()){
                    this.equipmentConfigureValue=chuanyunDeviceConfigureDTO.getPropertyvaluename();
                }
            }
        }
    }

    class DeviceInfoDTO {
        @JsonProperty("F0000001")
        private String equipmentCode;

        @JsonProperty("F0000002")
        private String equipmentMaterielCode;

        @JsonProperty("F0000003")
        private String projectId;

        public DeviceInfoDTO(ChuanyunDeviceDTO chuanyunDeviceDTO){
            if (null!=chuanyunDeviceDTO){
                if (null!=chuanyunDeviceDTO.getSncode()){
                    this.equipmentCode=chuanyunDeviceDTO.getSncode();
                }
                if(null!=chuanyunDeviceDTO.getItemcode()){
                    this.equipmentMaterielCode=chuanyunDeviceDTO.getItemcode();
                }
                if (null!=chuanyunDeviceDTO.getProjectcode()){
                    this.projectId=chuanyunDeviceDTO.getProjectcode();
                }
            }
        }
    }
}
